#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Project: Fable Input Output
#             https://github.com/silx-kit/fabio
#
#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
#
#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
"""
# Unit tests

# builds on stuff from ImageD11.test.testpeaksearch
28/11/2014
"""

import unittest
import os
import numpy
import shutil
import io
import logging
import fabio
from ...edfimage import edfimage
from ...fabioutils import GzipFile, BZ2File
from ..utilstest import UtilsTest
from ..testutils import LoggingValidator

logger = logging.getLogger(__name__)


class TestFlatEdfs(unittest.TestCase):
    """test some flat images"""

    def common_setup(self):
        self.BYTE_ORDER = "LowByteFirst" if numpy.little_endian else "HighByteFirst"
        self.MYHEADER = (
            "{\n%-1020s}\n"
            % (
                """Omega = 0.0 ;
                        Dim_1 = 256 ;
                        Dim_2 = 256 ;
                        DataType = FloatValue ;
                        ByteOrder = %s ;
                        Image = 1;
                        History-1 = something=something else;
                        \n\n"""
                % self.BYTE_ORDER
            )
        ).encode("latin-1")
        self.MYIMAGE = numpy.ones((256, 256), numpy.float32) * 10
        self.MYIMAGE[0, 0] = 0
        self.MYIMAGE[1, 1] = 20

        assert len(self.MYIMAGE[0:1, 0:1].tobytes()) == 4, self.MYIMAGE[
            0:1, 0:1
        ].tobytes()

    def setUp(self):
        """initialize"""
        self.common_setup()
        self.filename = os.path.join(UtilsTest.tempdir, "im0000.edf")
        if not os.path.isfile(self.filename):
            outf = open(self.filename, "wb")
            assert len(self.MYHEADER) % 1024 == 0
            outf.write(self.MYHEADER)
            outf.write(self.MYIMAGE.tobytes())
            outf.close()

        obj = edfimage()
        obj.read(self.filename)
        self.obj = obj

    def tearDown(self):
        self.obj.close()
        self.obj = None
        unittest.TestCase.tearDown(self)
        self.BYTE_ORDER = self.MYHEADER = self.MYIMAGE = None

    def test_read(self):
        """check readable"""
        self.assertEqual(
            self.obj.shape, (256, 256), msg="File %s has wrong shape " % self.filename
        )
        self.assertEqual(self.obj.bpp, 4, msg="bpp!=4 for file: %s" % self.filename)
        self.assertEqual(
            self.obj.bytecode,
            numpy.float32,
            msg="bytecode!=flot32 for file: %s" % self.filename,
        )
        self.assertEqual(
            self.obj.data.shape,
            (256, 256),
            msg="shape!=(256,256) for file: %s" % self.filename,
        )

    def test_getstats(self):
        """test statistics"""
        self.assertEqual(self.obj.getmean(), 10)
        self.assertEqual(self.obj.getmin(), 0)
        self.assertEqual(self.obj.getmax(), 20)

    def test_headers(self):
        self.assertEqual(len(self.obj.header), 7)
        expected_keys = [
            "Omega",
            "Dim_1",
            "Dim_2",
            "DataType",
            "ByteOrder",
            "Image",
            "History-1",
        ]
        self.assertEqual(expected_keys, list(self.obj.header.keys()))

        expected_values = {
            "Omega": "0.0",
            "Dim_1": "256",
            "Dim_2": "256",
            "DataType": "FloatValue",
            "Image": "1",
            "History-1": "something=something else",
        }
        for k, expected_value in expected_values.items():
            self.assertEqual(self.obj.header[k], expected_value)


class TestBzipEdf(TestFlatEdfs):
    """same for bzipped versions"""

    def setUp(self):
        """set it up"""
        TestFlatEdfs.setUp(self)
        if not os.path.isfile(self.filename + ".bz2"):
            with BZ2File(self.filename + ".bz2", "wb") as f:
                with open(self.filename, "rb") as d:
                    f.write(d.read())
        self.filename += ".bz2"


class TestGzipEdf(TestFlatEdfs):
    """same for gzipped versions"""

    def setUp(self):
        """set it up"""
        TestFlatEdfs.setUp(self)
        if not os.path.isfile(self.filename + ".gz"):
            with GzipFile(self.filename + ".gz", "wb") as f:
                with open(self.filename, "rb") as d:
                    f.write(d.read())
        self.filename += ".gz"


# statistics come from fit2d I think
# filename dim1 dim2 min max mean stddev
TESTIMAGES = """F2K_Seb_Lyso0675.edf     2048 2048 982 17467 1504.29  217.61
                F2K_Seb_Lyso0675.edf.bz2 2048 2048 982 17467 1504.29  217.61
                F2K_Seb_Lyso0675.edf.gz  2048 2048 982 17467 1504.29  217.61
                id13_badPadding.edf      512  512  85  61947 275.62   583.44 """


class TestEdfs(unittest.TestCase):
    """
    Read some test images
    """

    def setUp(self):
        self.im_dir = os.path.dirname(UtilsTest.getimage("F2K_Seb_Lyso0675.edf.bz2"))
        UtilsTest.getimage("id13_badPadding.edf.bz2")

    def test_read(self):
        """check we can read these images"""
        for line in TESTIMAGES.split("\n"):
            vals = line.split()
            name = vals[0]
            dim1, dim2 = [int(x) for x in vals[1:3]]
            shape = dim2, dim1
            mini, maxi, mean, stddev = [float(x) for x in vals[3:]]
            obj = edfimage()
            try:
                obj.read(os.path.join(self.im_dir, name))
            except Exception:
                logger.error("Cannot read image %s", name)
                raise
            self.assertAlmostEqual(
                mini, obj.getmin(), 2, "testedfs: %s getmin()" % name
            )
            self.assertAlmostEqual(maxi, obj.getmax(), 2, "testedfs: %s getmax" % name)
            logger.info("%s Mean: exp=%s, obt=%s" % (name, mean, obj.getmean()))
            self.assertAlmostEqual(
                mean, obj.getmean(), 2, "testedfs: %s getmean" % name
            )
            logger.info("%s StdDev:  exp=%s, obt=%s" % (name, stddev, obj.getstddev()))
            self.assertAlmostEqual(
                stddev, obj.getstddev(), 2, "testedfs: %s getstddev" % name
            )
            self.assertEqual(obj.shape, shape, "testedfs: %s shape" % name)
        obj = None

    def test_rebin(self):
        """test the rebin of edfdata"""
        f = edfimage()
        f.read(os.path.join(self.im_dir, "F2K_Seb_Lyso0675.edf"))
        f.rebin(1024, 1024)
        self.assertEqual(
            abs(numpy.array([[1547, 1439], [1536, 1494]]) - f.data).max(),
            0,
            "data are the same after rebin",
        )

    def tearDown(self):
        unittest.TestCase.tearDown(self)
        self.im_dir = None


class TestEdfCompressedData(unittest.TestCase):
    """
    Read some test images with their data-block compressed.
    Z-Compression and Gzip compression are implemented Bzip2 and byte offet are experimental
    """

    def setUp(self):
        self.im_dir = os.path.dirname(UtilsTest.getimage("edfGzip_U16.edf.bz2"))
        UtilsTest.getimage("edfCompressed_U16.edf.bz2")
        UtilsTest.getimage("edfUncompressed_U16.edf.bz2")

    def test_read(self):
        """check we can read these images"""
        ref = edfimage()
        gzipped = edfimage()
        compressed = edfimage()
        refFile = "edfUncompressed_U16.edf"
        gzippedFile = "edfGzip_U16.edf"
        compressedFile = "edfCompressed_U16.edf"

        ref.read(os.path.join(self.im_dir, refFile))
        gzipped.read(os.path.join(self.im_dir, gzippedFile))
        compressed.read(os.path.join(self.im_dir, compressedFile))

        self.assertEqual(
            (ref.data - gzipped.data).max(), 0, "Gzipped data block is correct"
        )
        self.assertEqual(
            (ref.data - compressed.data).max(),
            0,
            "Zlib compressed data block is correct",
        )


class TestEdfMultiFrame(unittest.TestCase):
    """
    Read some test images with their data-block compressed.
    Z-Compression and Gzip compression are implemented Bzip2 and byte offet are experimental
    """

    def setUp(self):
        self.multiFrameFilename = UtilsTest.getimage("MultiFrame.edf.bz2")[:-4]
        self.Frame0Filename = UtilsTest.getimage("MultiFrame-Frame0.edf.bz2")[:-4]
        self.Frame1Filename = UtilsTest.getimage("MultiFrame-Frame1.edf.bz2")[:-4]
        self.ref = edfimage()
        self.frame0 = edfimage()
        self.frame1 = edfimage()

        self.ref.read(self.multiFrameFilename)
        self.frame0.read(self.Frame0Filename)
        self.frame1.read(self.Frame1Filename)

    def tearDown(self):
        unittest.TestCase.tearDown(self)
        self.multiFrameFilename = self.Frame0Filename = self.Frame1Filename = (
            self.ref
        ) = self.frame0 = self.frame1 = None

    def test_getFrame_multi(self):
        self.assertEqual(
            (self.ref.data - self.frame0.data).max(),
            0,
            "getFrame_multi: Same data for frame 0",
        )
        f1_multi = self.ref.getframe(1)
        # logger.warning("f1_multi.header=%s\nf1_multi.data=  %s" % (f1_multi.header, f1_multi.data))
        self.assertEqual(
            (f1_multi.data - self.frame1.data).max(),
            0,
            "getFrame_multi: Same data for frame 1",
        )

    def test_getFrame_mono(self):
        self.assertEqual(
            (self.ref.data - self.frame0.data).max(),
            0,
            "getFrame_mono: Same data for frame 0",
        )
        f1_mono = self.frame0.getframe(1)
        self.assertEqual(
            (f1_mono.data - self.frame1.data).max(),
            0,
            "getFrame_mono: Same data for frame 1",
        )

    def test_next_multi(self):
        self.assertEqual(
            (self.ref.data - self.frame0.data).max(),
            0,
            "next_multi: Same data for frame 0",
        )
        next_ = self.ref.next()
        self.assertEqual(
            (next_.data - self.frame1.data).max(),
            0,
            "next_multi: Same data for frame 1",
        )

    def text_next_mono(self):
        self.assertEqual(
            (self.ref.data - self.frame0.data).max(),
            0,
            "next_mono: Same data for frame 0",
        )
        next_ = self.frame0.next()
        self.assertEqual(
            (next_.data - self.frame1.data).max(), 0, "next_mono: Same data for frame 1"
        )

    def test_previous_multi(self):
        f1 = self.ref.getframe(1)
        self.assertEqual(
            (f1.data - self.frame1.data).max(),
            0,
            "previous_multi: Same data for frame 1",
        )
        f0 = f1.previous()
        self.assertEqual(
            (f0.data - self.frame1.data).max(),
            0,
            "previous_multi: Same data for frame 0",
        )

    def test_previous_mono(self):
        f1 = self.ref.getframe(1)
        self.assertEqual(
            (f1.data - self.frame1.data).max(),
            0,
            "previous_mono: Same data for frame 1",
        )
        prev = self.frame1.previous()
        self.assertEqual(
            (prev.data - self.frame0.data).max(),
            0,
            "previous_mono: Same data for frame 0",
        )

    def test_openimage_multiframes(self):
        "test if openimage can directly read first or second frame of a multi-frame"
        self.assertEqual(
            (fabio.open(self.multiFrameFilename).data - self.frame0.data).max(),
            0,
            "openimage_multiframes: Same data for default ",
        )
        # print(fabio.open(self.multiFrameFilename, 0).data)
        self.assertEqual(
            (fabio.open(self.multiFrameFilename, 0).data - self.frame0.data).max(),
            0,
            "openimage_multiframes: Same data for frame 0",
        )
        self.assertEqual(
            (fabio.open(self.multiFrameFilename, 1).data - self.frame1.data).max(),
            0,
            "openimage_multiframes: Same data for frame 1",
        )


class TestEdfFastRead(unittest.TestCase):
    """
    Read some test images with their data-block compressed.
    Z-Compression and Gzip compression are implemented Bzip2 and byte offet are experimental
    """

    def setUp(self):
        self.refFilename = UtilsTest.getimage("MultiFrame-Frame0.edf.bz2")
        self.fastFilename = self.refFilename[:-4]

    def test_fastread(self):
        ref = fabio.open(self.refFilename)
        refdata = ref.data
        obt = ref.fast_read_data(self.fastFilename)
        self.assertEqual(abs(obt - refdata).max(), 0, "testedffastread: Same data")

    def test_578(self):
        "Non regression for issue 578: EDFImage.fast_read_data: cannot read a line"

        file_name = os.path.join(UtilsTest.tempdir, "test_578.edf")
        # create some dummy data
        edf_writer = fabio.edfimage.EdfImage(
            data=numpy.arange(0, 99).reshape(9, 11),
        )
        edf_writer.write(file_name)

        edf_reader = fabio.open(file_name)

        # if I want to read it line by line
        line_index_to_read = 4
        line_data = edf_reader.fast_read_roi(
            file_name,
            coords=(
                slice(3, 9),
                slice(line_index_to_read, line_index_to_read + 1),
            ),
        )
        self.assertTrue(
            numpy.allclose(
                edf_writer.data[3:9, line_index_to_read : line_index_to_read + 1],
                line_data,
            )
        )


class TestEdfWrite(unittest.TestCase):
    """
    Write dummy edf files with various compression schemes
    """

    tmpdir = UtilsTest.tempdir

    def setUp(self):
        self.data = numpy.arange(100).reshape((10, 10))
        self.header = {"toto": "tutu"}

    def testFlat(self):
        self.filename = os.path.join(self.tmpdir, "merged.azim")
        e = edfimage(data=self.data, header=self.header)
        e.write(self.filename)
        r = fabio.open(self.filename)
        self.assertTrue(r.header["toto"] == self.header["toto"], "header are OK")
        self.assertTrue(abs(r.data - self.data).max() == 0, "data are OK")
        self.assertEqual(
            int(r.header["EDF_HeaderSize"]), 512, "header size is one 512 block"
        )

    def testGzip(self):
        self.filename = os.path.join(self.tmpdir, "merged.azim.gz")
        e = edfimage(data=self.data, header=self.header)
        e.write(self.filename)
        r = fabio.open(self.filename)
        self.assertTrue(r.header["toto"] == self.header["toto"], "header are OK")
        self.assertTrue(abs(r.data - self.data).max() == 0, "data are OK")
        self.assertEqual(
            int(r.header["EDF_HeaderSize"]), 512, "header size is one 512 block"
        )

    def testBzip2(self):
        self.filename = os.path.join(self.tmpdir, "merged.azim.gz")
        e = edfimage(data=self.data, header=self.header)
        e.write(self.filename)
        r = fabio.open(self.filename)
        self.assertTrue(r.header["toto"] == self.header["toto"], "header are OK")
        self.assertTrue(abs(r.data - self.data).max() == 0, "data are OK")
        self.assertEqual(
            int(r.header["EDF_HeaderSize"]), 512, "header size is one 512 block"
        )

    def tearDown(self):
        os.unlink(self.filename)


class TestEdfRegression(unittest.TestCase):
    """
    Test suite to prevent regression
    """

    def test_bug_27(self):
        """
        import fabio
        obj = fabio.open("any.edf")
        obj.header["missing"]="blah"
        obj.write("any.edf")
        """
        # create dummy image:
        shape = (32, 32)
        data = (
            numpy.random.randint(0, 6500, size=shape[0] * shape[1])
            .astype("uint16")
            .reshape(shape)
        )
        fname = os.path.join(UtilsTest.tempdir, "bug27.edf")
        e = edfimage(data=data, header={"key1": "value1"})
        e.write(fname)
        del e

        obj = fabio.open(fname)
        obj.header["missing"] = "blah"
        obj.write(fname)

        del obj

    def test_remove_metadata_header(self):
        filename = UtilsTest.getimage("face.edf.bz2")[0:-4]
        output_filename = os.path.join(
            UtilsTest.tempdir, "test_remove_metadata_header.edf"
        )

        image = fabio.open(filename)
        del image.header["Dim_1"]
        image.write(output_filename)
        image2 = fabio.open(output_filename)
        self.assertEqual(image.shape, image2.shape)

    def test_bug_459(self):
        h = {
            "HeaderID": "EH:000001:000000:000000",
            "Image": "1",
            "ByteOrder": "LowByteFirst",
            "DataType": "Float",
            "Dim_1": "3216",
            "Dim_2": "3000",
            "Size": "38592000",
            "Date": "30-Oct-2021",
            "Lcurve": "0",
            "Mh": "24.996",
            "Mv": "24.996",
            "angles_fname": "angles_file.txt",
            "angles_sign": "-1",
            "ans": ";",
            "approach": "8",
            "argn": "[0,",
            "avg_plane_zero": "6.2031",
            "axisfilesduringscan": "1",
            "axisposition": "global",
            "betash": "0.0179803",
            "betasv": "0.0179803",
            "calc_residu": "0",
            "centralpart": "2048",
            "centralpart_shift_h": "0",
            "centralpart_shift_v": "0",
            "centralpart_struct": ";",
            "check_spectrum": "0",
            "check_z1v": "0",
            "constrain": "0",
            "correct_detector": "3",
            "correct_distortion_par": ";",
            "correct_shrink": "0",
            "correct_shrink_positive": "0",
            "correct_whitefield_par": "17",
            "cutn": "0.0511098",
            "cylinder": "0",
            "debug": "0",
            "delta_beta": "2119.4",
            "delta_beta_normalised": "0",
            "delta_beta_range": "2119.4",
            "delta_beta_test": "0",
            "dim_h": "2048",
            "dim_v": "2048",
            "dir": "/data/visitor/ls3023/id16a/PR3/",
            "direc": "",
        }
        with LoggingValidator(fabio.edfimage.logger, error=0, warning=0):
            fabio.edfimage.EdfFrame.get_data_rank(h)


class TestBadFiles(unittest.TestCase):
    filename_template = "%s.edf"

    @classmethod
    def setUpClass(cls):
        cls.tmp_directory = os.path.join(UtilsTest.tempdir, cls.__name__)
        os.makedirs(cls.tmp_directory)
        cls.create_resources()

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.tmp_directory)

    @classmethod
    def create_resources(cls):
        filename = os.path.join(cls.tmp_directory, cls.filename_template % "base")
        cls.base_filename = filename
        with io.open(filename, "wb") as fd:
            cls.write_header(fd, 1)
            cls.header1 = fd.tell()
            cls.write_data(fd)
            cls.data1 = fd.tell()
            cls.write_header(fd, 2)
            cls.header2 = fd.tell()
            cls.write_data(fd)
            cls.data2 = fd.tell()

    @classmethod
    def write_header(cls, fd, image_number):
        byte_order = "LowByteFirst" if numpy.little_endian else "HighByteFirst"
        byte_order = byte_order.encode("latin-1")

        fd.write(b"{\n")
        fd.write(b"Omega = 0.0 ;\n")
        fd.write(b"Dim_1 = 256 ;\n")
        fd.write(b"Dim_2 = 256 ;\n")
        fd.write(b"DataType = FloatValue ;\n")
        fd.write(b"ByteOrder = %s ;\n" % byte_order)
        fd.write(b"Image = %d ;\n" % image_number)
        fd.write(b"History-1 = something=something else;\n")
        fd.write(b"}\n")

    @classmethod
    def write_data(cls, fd):
        data = numpy.ones((256, 256), numpy.float32) * 10
        data[0, 0] = 0
        data[1, 1] = 20
        fd.write(data.tobytes())

    @classmethod
    def copy_base(cls, filename, size):
        with io.open(cls.base_filename, "rb") as fd_base:
            with io.open(filename, "wb") as fd_result:
                fd_result.write(fd_base.read(size))

    @classmethod
    def open(cls, filename):
        image = fabio.edfimage.EdfImage()
        image.read(filename)
        return image

    def test_base(self):
        filename = os.path.join(
            self.tmp_directory, self.filename_template % str(self.id())
        )
        size = self.data2
        self.copy_base(filename, size)

        image = self.open(filename)
        self.assertEqual(image.nframes, 2)

        frame = image.getframe(0)
        self.assertEqual(frame.header["Image"], "1")
        self.assertEqual(frame.data[-1].sum(), 2560)
        frame = image.getframe(1)
        self.assertEqual(frame.header["Image"], "2")
        self.assertEqual(frame.data[-1].sum(), 2560)

    def test_empty(self):
        filename = os.path.join(
            self.tmp_directory, self.filename_template % str(self.id())
        )
        f = io.open(filename, "wb")
        f.close()

        self.assertRaises(IOError, self.open, filename)

    def test_wrong_magic(self):
        filename = os.path.join(
            self.tmp_directory, self.filename_template % str(self.id())
        )
        f = io.open(filename, "wb")
        f.write(b"\x10\x20\x30")
        f.close()

        self.assertRaises(IOError, self.open, filename)

    def test_half_header(self):
        filename = os.path.join(
            self.tmp_directory, self.filename_template % str(self.id())
        )
        size = self.header1 // 2
        self.copy_base(filename, size)

        self.assertRaises(IOError, self.open, filename)

    def test_header_with_no_data(self):
        filename = os.path.join(
            self.tmp_directory, self.filename_template % str(self.id())
        )
        size = self.header1
        self.copy_base(filename, size)

        image = self.open(filename)
        self.assertIn(image.nframes, [0, 1])
        self.assertTrue(image.incomplete_file)

    def test_header_with_half_data(self):
        filename = os.path.join(
            self.tmp_directory, self.filename_template % str(self.id())
        )
        size = (self.header1 + self.data1) // 2
        self.copy_base(filename, size)

        image = self.open(filename)
        self.assertEqual(image.nframes, 1)
        self.assertTrue(image.incomplete_file)

        frame = image
        self.assertEqual(frame.header["Image"], "1")
        self.assertEqual(frame.data[-1].sum(), 0)
        self.assertTrue(frame.incomplete_data)

    def test_full_frame_plus_half_header(self):
        filename = os.path.join(
            self.tmp_directory, self.filename_template % str(self.id())
        )
        size = (self.data1 + self.header2) // 2
        self.copy_base(filename, size)

        image = self.open(filename)
        self.assertEqual(image.nframes, 1)
        self.assertTrue(image.incomplete_file)

        frame = image
        self.assertEqual(frame.header["Image"], "1")
        self.assertEqual(frame.data[-1].sum(), 2560)
        self.assertFalse(frame.incomplete_data)

    def test_full_frame_plus_header_with_no_data(self):
        filename = os.path.join(
            self.tmp_directory, self.filename_template % str(self.id())
        )
        size = self.header2
        self.copy_base(filename, size)

        image = self.open(filename)
        self.assertIn(image.nframes, [1, 2])
        self.assertTrue(image.incomplete_file)

        frame = image
        self.assertEqual(frame.header["Image"], "1")
        self.assertEqual(frame.data[-1].sum(), 2560)
        self.assertFalse(frame.incomplete_data)

    def test_full_frame_plus_header_with_half_data(self):
        filename = os.path.join(
            self.tmp_directory, self.filename_template % str(self.id())
        )
        size = (self.header2 + self.data2) // 2
        self.copy_base(filename, size)

        image = self.open(filename)
        self.assertEqual(image.nframes, 2)
        self.assertTrue(image.incomplete_file)

        frame = image.getframe(0)
        self.assertEqual(frame.header["Image"], "1")
        self.assertEqual(frame.data[-1].sum(), 2560)
        self.assertFalse(frame.incomplete_data)

        frame = image.getframe(1)
        self.assertEqual(frame.header["Image"], "2")
        self.assertEqual(frame.data[-1].sum(), 0)
        self.assertTrue(frame.incomplete_data)


class TestBadGzFiles(TestBadFiles):
    filename_template = "%s.edf.gz"

    @classmethod
    def write_header(cls, fd, image_number):
        with GzipFile(fileobj=fd, mode="wb") as gzfd:
            TestBadFiles.write_header(gzfd, image_number)

    @classmethod
    def write_data(cls, fd):
        with GzipFile(fileobj=fd, mode="wb") as gzfd:
            TestBadFiles.write_data(gzfd)


class TestSphere2SaxsSamples(unittest.TestCase):
    """Test some samples from sphere2saxs"""

    def setUp(self):
        unittest.TestCase.setUp(self)
        self.samples = UtilsTest.resources.getdir("sphere2saxs_output.tar.bz2")

    SAMPLES = {
        "multi.edf": (
            5,
            (200, 100),
            numpy.float32,
            (6.292408e-05, 0.5594252, 3.2911296, 0.82902604),
        ),
        "multi.edf.gz": (
            5,
            (200, 100),
            numpy.float32,
            (6.292408e-05, 0.5594252, 3.2911296, 0.82902604),
        ),
        "sphere.edf": (
            1,
            (200, 100),
            numpy.float32,
            (6.292408e-05, 0.5594252, 3.2911296, 0.82902604),
        ),
        "sphere.edf.gz": (
            1,
            (200, 100),
            numpy.float32,
            (6.292408e-05, 0.5594252, 3.2911296, 0.82902604),
        ),
    }

    def test_all_images(self):
        for filename in self.samples:
            if not os.path.isfile(filename):
                continue
            with fabio.open(filename) as f:
                logger.debug("Reading file %s", filename)
                expected_data = self.SAMPLES[os.path.basename(filename)]
                nframes, shape, dtype, datainfo = expected_data
                self.assertEqual(f.nframes, nframes)
                self.assertEqual(f.shape, shape)
                self.assertEqual(f.dtype, dtype)
                vmin, vmean, vmax, vstd = datainfo
                self.assertEqual(f.dtype, dtype)
                self.assertAlmostEqual(f.data.min(), vmin, places=4)
                self.assertAlmostEqual(f.data.mean(), vmean, places=4)
                self.assertAlmostEqual(f.data.max(), vmax, places=4)
                self.assertAlmostEqual(f.data.std(), vstd, places=4)


class TestEdfIterator(unittest.TestCase):
    """Read different EDF files with lazy iterator"""

    def test_multi_frame(self):
        """Test iterator on a multi-frame EDF"""
        filename = UtilsTest.getimage("MultiFrame.edf.bz2")

        iterator = fabio.edfimage.EdfImage.lazy_iterator(filename)
        ref = fabio.open(filename)

        for index in range(ref.nframes):
            frame = next(iterator)
            ref_frame = ref.getframe(index)
            self.assertEqual(
                numpy.abs(ref_frame.data - frame.data).max(),
                0,
                "Test frame %d data" % index,
            )
            self.assertEqual(
                ref_frame.header, frame.header, "Test frame %d header" % index
            )

        with self.assertRaises(StopIteration):
            next(iterator)

    def test_single_frame(self):
        """Test iterator on a single frame EDF"""
        filename = UtilsTest.getimage("edfCompressed_U16.edf")
        iterator = fabio.edfimage.EdfImage.lazy_iterator(filename)

        frame = next(iterator)
        ref = fabio.open(filename)
        self.assertEqual((ref.data - frame.data).max(), 0, "Test data")
        self.assertEqual(ref.header, frame.header, "Test header")

        with self.assertRaises(StopIteration):
            next(iterator)


class TestEdfBadHeader(unittest.TestCase):
    """Test reader behavior with corrupted header file"""

    def setUp(self):
        self.fgood = os.path.join(UtilsTest.tempdir, "TestEdfGoodHeaderPadding.edf")
        self.fbad = os.path.join(UtilsTest.tempdir, "TestEdfBadHeaderPadding.edf")
        self.fzero = os.path.join(UtilsTest.tempdir, "TestEdfZeroHeaderPadding.edf")
        self.fnonascii = os.path.join(UtilsTest.tempdir, "TestEdfNonAsciiItem.edf")
        self.data = numpy.zeros((10, 11), numpy.uint8)
        self.hdr = {"mykey": "myvalue", "title": "ok"}

        good = fabio.edfimage.edfimage(self.data, self.hdr)
        good.write(self.fgood)
        with fabio.open(self.fgood) as good:
            self.good_header = good.header

        with open(self.fgood, "rb") as fh:
            hdr = bytearray(fh.read(512))
            while hdr.find(b"}") < 0:
                hdr += fh.read(512)
            data = fh.read()
        with open(self.fbad, "wb") as fb:
            start = hdr.rfind(b";") + 1
            end = hdr.find(b"}") - 1
            hdr[start:end] = [ord("\n")] + [0xCD] * (end - start - 1)
            fb.write(hdr)
            fb.write(data)
        with open(self.fzero, "wb") as fb:
            # insert some 0x00 to be stripped
            key = b"myvalue"
            z = hdr.find(key)
            hdr[z + len(key)] = 0
            fb.write(hdr)
            fb.write(data)
        with open(self.fnonascii, "wb") as fb:
            hdr[z : z + 1] = 0xC3, 0xA9  # e-acute in utf-8 ??
            with open(self.fnonascii, "wb") as fb:
                fb.write(hdr)
                fb.write(data)

    def tearDown(self):
        os.remove(self.fgood)
        os.remove(self.fbad)
        os.remove(self.fzero)
        os.remove(self.fnonascii)

    def testReadBadPadding(self):
        """
        Some old data were found with headers padded with 0xcd (issue #373)
        """
        with fabio.open(self.fbad) as im:
            self.assertTrue((im.data == 0).all())
            self.assertEqual(im.header, self.good_header)

    def testReadGoodPadding(self):
        with fabio.open(self.fgood) as im:
            self.assertTrue((im.data == 0).all())
            self.assertEqual(im.header, self.good_header)

    def testReadZeroPadding(self):
        with fabio.open(self.fzero) as im:
            self.assertTrue((im.data == 0).all())
            self.assertEqual(im.header, self.good_header)

    def testNonAsciiHeader(self):
        """Non-ascii characters are skipped."""
        with fabio.open(self.fnonascii) as im:
            self.assertTrue((im.data == 0).all())
            expected = dict(self.good_header)
            expected.pop("mykey")
            self.assertEqual(im.header, expected)


def suite():
    loadTests = unittest.defaultTestLoader.loadTestsFromTestCase
    testsuite = unittest.TestSuite()
    testsuite.addTest(loadTests(TestFlatEdfs))
    testsuite.addTest(loadTests(TestBzipEdf))
    testsuite.addTest(loadTests(TestGzipEdf))
    testsuite.addTest(loadTests(TestEdfs))
    testsuite.addTest(loadTests(TestEdfCompressedData))
    testsuite.addTest(loadTests(TestEdfMultiFrame))
    testsuite.addTest(loadTests(TestEdfFastRead))
    testsuite.addTest(loadTests(TestEdfWrite))
    testsuite.addTest(loadTests(TestEdfRegression))
    testsuite.addTest(loadTests(TestBadFiles))
    testsuite.addTest(loadTests(TestBadGzFiles))
    testsuite.addTest(loadTests(TestEdfIterator))
    testsuite.addTest(loadTests(TestSphere2SaxsSamples))
    testsuite.addTest(loadTests(TestEdfBadHeader))
    return testsuite


if __name__ == "__main__":
    runner = unittest.TextTestRunner()
    runner.run(suite())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Project: Fable Input Output
#             https://github.com/silx-kit/fabio
#
#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
#
#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#

"""
Image provided by Henning O. Sørensen
"""

__date__ = "27/10/2025"
__author__ = "Jérôme Kieffer"

import unittest
import os
import logging
import fabio
from fabio.HiPiCimage import HipicImage
from ..utilstest import UtilsTest

logger = logging.getLogger(__name__)

# filename dim1 dim2 min max mean stddev
TESTIMAGES = [
    (
        "Radiograph_SP8_BL47XU_HiPicimage.img",
        (2048, 2048),
        2029,
        46937,
        22125.5196352005,
        9089.959313208194,
    ),
    (
        "Radiograph_SP8_BL47XU_HiPicimage.img.gz",
        (2048, 2048),
        2029,
        46937,
        22125.5196352005,
        9089.959313208194,
    ),
    (
        "Radiograph_SP8_BL47XU_HiPicimage.img.bz2",
        (2048, 2048),
        2029,
        46937,
        22125.5196352005,
        9089.959313208194,
    ),
]


class TestHiPiCImage(unittest.TestCase):
    """ """

    def setUp(self):
        """Download images"""
        self.im_dir = os.path.dirname(
            UtilsTest.getimage("Radiograph_SP8_BL47XU_HiPicimage.img.bz2")
        )

    def test_read(self):
        """check we can read dm3 images"""
        for info in TESTIMAGES:
            name, shape, mini, maxi, mean, stddev = info
            fname = os.path.join(self.im_dir, name)
            obj1 = HipicImage()
            obj1.read(fname)
            obj2 = fabio.open(fname)
            for obj in (obj1, obj2):
                self.assertAlmostEqual(mini, obj.getmin(), 2, "getmin")
                self.assertAlmostEqual(maxi, obj.getmax(), 2, "getmax")
                got_mean = obj.getmean()
                self.assertAlmostEqual(
                    mean, got_mean, 2, "getmean exp %s != got %s" % (mean, got_mean)
                )
                self.assertAlmostEqual(stddev, obj.getstddev(), 2, "getstddev")
                self.assertEqual(shape, obj.shape)


def suite():
    loadTests = unittest.defaultTestLoader.loadTestsFromTestCase
    testsuite = unittest.TestSuite()
    testsuite.addTest(loadTests(TestHiPiCImage))
    return testsuite


if __name__ == "__main__":
    runner = unittest.TextTestRunner()
    runner.run(suite())



# coding: utf-8
#
#    Project: X-ray image reader
#             https://github.com/silx-kit/fabio
#
#
#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
#
#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#  .
#  The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#  .
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#  THE SOFTWARE.
"""
Test cases for the Next/Previous ...

28/11/2014
"""

import unittest
import logging
import fabio

logger = logging.getLogger(__name__)


class TestNext(unittest.TestCase):
    def test_next1(self):
        files = [
            ["data0001.edf", "data0002.edf"],
            ["bob1.edf", "bob2.edf"],
            ["1.edf", "2.edf"],
            ["1.mar2300", "2.mar2300"],
        ]
        for name, next_ in files:
            self.assertEqual(next_, fabio.next_filename(name))


class TestPrev(unittest.TestCase):
    def test_prev1(self):
        files = [
            ["data0001.edf", "data0000.edf"],
            ["bob1.edf", "bob0.edf"],
            ["1.edf", "0.edf"],
            ["1.mar2300", "0.mar2300"],
        ]
        for name, prev in files:
            self.assertEqual(prev, fabio.previous_filename(name))


class TestJump(unittest.TestCase):
    def test_jump1(self):
        files = [
            ["data0001.edf", "data99993.edf", 99993],
            ["bob1.edf", "bob0.edf", 0],
            ["1.edf", "123456.edf", 123456],
            ["mydata001.mar2300.gz", "mydata003.mar2300.gz", 3],
        ]
        for name, res, num in files:
            self.assertEqual(res, fabio.jump_filename(name, num))


def suite():
    loadTests = unittest.defaultTestLoader.loadTestsFromTestCase
    testsuite = unittest.TestSuite()
    testsuite.addTest(loadTests(TestNext))
    testsuite.addTest(loadTests(TestPrev))
    testsuite.addTest(loadTests(TestJump))
    return testsuite


if __name__ == "__main__":
    runner = unittest.TextTestRunner()
    runner.run(suite())



#!/usr/bin/env python
# coding: utf-8
#
#    Project: X-ray image reader
#             https://github.com/silx-kit/fabio
#
#
#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
#
#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE

"""
Authors: Henning O. Sorensen & Erik Knudsen
         Center for Fundamental Research: Metal Structures in Four Dimensions
         Risoe National Laboratory
         Frederiksborgvej 399
         DK-4000 Roskilde
         email:erik.knudsen@risoe.dk

        + Jon Wright, ESRF

Information about the file format from Masakatzu Kobayashi is highly appreciated
"""

import numpy
import logging
from .fabioimage import FabioImage

logger = logging.getLogger(__name__)


class HipicImage(FabioImage):
    """Read HiPic images e.g. collected with a Hamamatsu CCD camera"""

    DESCRIPTION = "HiPic file format from Hamamatsu CCD cameras"

    DEFAULT_EXTENSIONS = ["img"]

    def _readheader(self, infile):
        """
        Read in a header from an already open file

        """
        Image_tag = infile.read(2).decode()
        Comment_len = numpy.frombuffer(infile.read(2), numpy.uint16)
        Dim_1 = numpy.frombuffer(infile.read(2), numpy.uint16)[0]
        Dim_2 = numpy.frombuffer(infile.read(2), numpy.uint16)[0]
        Dim_1_offset = numpy.frombuffer(infile.read(2), numpy.uint16)[0]
        Dim_2_offset = numpy.frombuffer(infile.read(2), numpy.uint16)[0]
        _HeaderType = numpy.frombuffer(infile.read(2), numpy.uint16)[0]
        _Dump = infile.read(50)
        Comment = infile.read(Comment_len[0])
        Comment = Comment.decode()
        self.header["Image_tag"] = Image_tag
        self.header["Dim_1"] = Dim_1
        self.header["Dim_2"] = Dim_2
        self.header["Dim_1_offset"] = Dim_1_offset
        self.header["Dim_2_offset"] = Dim_2_offset
        # self.header['Comment'] = Comment
        if Image_tag != "IM":
            # This does not look like an HiPic file
            logger.warning(
                "No opening. Corrupt header of HiPic file %s", str(infile.name)
            )
        Comment_split = Comment[: Comment.find("\x00")].split("\r\n")

        for topcomment in Comment_split:
            topsplit = topcomment.split(",")
            for line in topsplit:
                if "=" in line:
                    key, val = line.split("=", 1)
                    # Users cannot type in significant whitespace
                    key = key.rstrip().lstrip()
                    self.header_keys.append(key)
                    self.header[key] = val.lstrip().rstrip()
                    self.header[key] = val.lstrip('"').rstrip('"')

    def read(self, fname, frame=None):
        """
        Read in header into self.header and
            the data   into self.data
        """
        self.header = self.check_header()
        self.resetvals()
        infile = self._open(fname, "rb")
        self._readheader(infile)
        # Compute image size
        try:
            dim1 = int(self.header["Dim_1"])
            dim2 = int(self.header["Dim_2"])
            self._shape = dim2, dim1
        except (ValueError, KeyError):
            raise IOError("HiPic file %s is corrupted, cannot read it" % str(fname))
        dtype = numpy.dtype(numpy.uint16)
        self._dtype = dtype

        # Read image data
        block = infile.read(dim1 * dim2 * dtype.itemsize)
        infile.close()

        # now read the data into the array
        try:
            self.data = numpy.frombuffer(block, dtype).copy().reshape((dim2, dim1))
        except Exception:
            logger.debug("%s %s %s %s %s", len(block), dtype, self.bpp, dim2, dim1)
            logger.debug("Backtrace", exc_info=True)
            raise IOError(
                "Size spec in HiPic-header does not match size of image data field"
            )
        self._dtype = None
        self._shape = None

        #### The case below is not true for data collected at
        #### BL47XU/BL20XU/BL20B at SPring-8 - here the data is saved as
        #### 16 bit - so values above 4095 should be negative.
        #### Therefore I have now commented it out.

        # # Sometimes these files are not saved as 12 bit,
        # # But as 16 bit after bg subtraction - which results
        # # negative values saved as 16bit. Therefore values higher
        # # 4095 is really negative values
        # if self.data.max() > 4095:
        #     gt12bit = self.data > 4095
        #     self.data = self.data - gt12bit * (2 ** 16 - 1)
        return self


HiPiCimage = HipicImage



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Project: Fable Input Output
#             https://github.com/silx-kit/fabio
#
#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
#
#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#  .
#  The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#  .
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#  THE SOFTWARE.
"""
# Unit tests

# builds on stuff from ImageD11.test.testpeaksearch
28/11/2014
"""

import unittest
import logging
import fabio
from .. import fabioformats
from ..utils import deprecation
from ..utils import testutils

logger = logging.getLogger(__name__)


class TestRegistration(unittest.TestCase):
    def test_fabio_factory(self):
        image = fabio.factory("edfimage")
        self.assertIsNotNone(image)

    def test_fabio_factory_missing_format(self):
        self.assertRaises(RuntimeError, fabio.factory, "foobarimage")

    def test_fabioformats_factory(self):
        image = fabioformats.factory("edfimage")
        self.assertIsNotNone(image)

    def test_fabioformats_factory_missing_format(self):
        self.assertRaises(RuntimeError, fabioformats.factory, "foobarimage")

    @testutils.test_logging(deprecation.depreclog, warning=1)
    def test_deprecated_fabioimage_factory(self):
        """Check that it is still working"""
        image = fabio.fabioimage.FabioImage.factory("edfimage")
        self.assertIsNotNone(image)

    @testutils.test_logging(deprecation.depreclog, warning=1)
    def test_deprecated_fabioimage_factory_missing_format(self):
        """Check that it is still working"""
        self.assertRaises(
            RuntimeError, fabio.fabioimage.FabioImage.factory, "foobarimage"
        )

    def test_not_existing(self):
        self.assertIsNone(fabioformats.get_class_by_name("myformat0"))

    def test_annotation(self):
        @fabio.register
        class MyFormat1(fabio.fabioimage.FabioImage):
            pass

        self.assertIsNotNone(fabioformats.get_class_by_name("myformat1"))

    def test_function(self):
        class MyFormat2(fabio.fabioimage.FabioImage):
            pass

        fabio.register(MyFormat2)
        self.assertIsNotNone(fabioformats.get_class_by_name("myformat2"))

    def test_extenstion_registry(self):
        for ext in fabioformats._get_extension_mapping():
            self.assertFalse("." in ext)


def suite():
    loadTests = unittest.defaultTestLoader.loadTestsFromTestCase
    testsuite = unittest.TestSuite()
    testsuite.addTest(loadTests(TestRegistration))
    return testsuite


if __name__ == "__main__":
    runner = unittest.TextTestRunner()
    runner.run(suite())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Project: X-ray image reader
#             https://github.com/silx-kit/fabio
#
#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
#
#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#  .
#  The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#  .
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#  THE SOFTWARE.
"""Unit tests for nexus file reader"""

import unittest
import os
import logging
from .utilstest import UtilsTest
from .. import nexus

logger = logging.getLogger(__name__)


class TestNexus(unittest.TestCase):
    def setUp(self):
        if nexus.h5py is None:
            self.skipTest("h5py library is not available. Skipping Nexus test")

    def test_nexus(self):
        "Test creation of Nexus files"
        fname = os.path.join(UtilsTest.tempdir, "nexus1.h5")
        nex = nexus.Nexus(fname)
        entry = nex.new_entry("entry")
        nex.new_instrument(entry, "ID00")
        nex.new_detector("camera")
        self.assertEqual(len(nex.get_entries()), 2, "nexus file has 2 entries")
        nex.close()
        self.assertTrue(os.path.exists(fname))
        os.unlink(fname)

    def test_from_time(self):
        fname = os.path.join(UtilsTest.tempdir, "nexus2.h5")
        nex = nexus.Nexus(fname)
        entry = nex.new_entry("entry")
        time1 = nexus.from_isotime(entry["start_time"][()])
        ltime = [str(entry["start_time"][()]).encode()]
        entry.create_dataset("bad_time", data=ltime)
        time2 = nexus.from_isotime(entry["bad_time"][()])
        self.assertEqual(time1, time2, "start_time in list does not works !")
        nex.close()
        self.assertTrue(os.path.exists(fname))
        os.unlink(fname)


def suite():
    loadTests = unittest.defaultTestLoader.loadTestsFromTestCase
    testsuite = unittest.TestSuite()
    testsuite.addTest(loadTests(TestNexus))
    return testsuite


if __name__ == "__main__":
    runner = unittest.TextTestRunner()
    runner.run(suite())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Project: Fable Input Output
#             https://github.com/silx-kit/fabio
#
#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
#
#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
"""
Test JPEG 2000 format
"""

import unittest
import numpy
import logging

try:
    from PIL import Image
except ImportError:
    Image = None
import fabio
from ... import jpeg2kimage
from ..utilstest import UtilsTest

logger = logging.getLogger(__name__)


def isPilUsable():
    print("jpeg2kimage.PIL:", jpeg2kimage.PIL)
    if jpeg2kimage.PIL is None:
        return None
    try:
        if hasattr(jpeg2kimage.PIL.Image, "frombytes"):
            frombytes = jpeg2kimage.PIL.Image.frombytes
        else:
            frombytes = jpeg2kimage.PIL.Image.frombuffer
        frombytes("1", (2, 2), b"", decoder_name="jpeg2k")
    except Exception as e:
        if e.args[0] == "decoder jpeg2k not available":
            return False
        # Skip decoding error
    return True


def isGlymurUsable():
    if jpeg2kimage.glymur is None:
        return None
    import glymur

    if tuple(glymur.version.openjpeg_version_tuple) < (1, 5, 0):
        return False
    return True


class TestJpeg2KImage(unittest.TestCase):
    """Test the class format"""

    def setUp(self):
        if not (isPilUsable() or isGlymurUsable()):
            self.skipTest("nor PIL neither glymur are available")

    def loadImage(self, filename):
        image_format = jpeg2kimage.Jpeg2KImage()
        image = image_format.read(filename)
        return image

    def test_open_uint8(self):
        filename = "binned_data_uint8.jp2"
        filename = UtilsTest.getimage(filename + ".bz2")[:-4]
        image = self.loadImage(filename)
        self.assertEqual(image.data.shape, (120, 120))
        self.assertEqual(image.data.dtype, numpy.uint8)

    def test_open_uint16(self):
        filename = "binned_data_uint16.jp2"
        filename = UtilsTest.getimage(filename + ".bz2")[:-4]
        image_format = jpeg2kimage.Jpeg2KImage()
        image = image_format.read(filename)
        self.assertEqual(image.data.shape, (120, 120))
        self.assertEqual(image.data.dtype, numpy.uint16)

    def test_open_wrong_format(self):
        filename = "MultiFrame.edf"
        filename = UtilsTest.getimage(filename + ".bz2")[:-4]
        image_format = jpeg2kimage.Jpeg2KImage()

        try:
            _image = image_format.read(filename)
            self.fail()
        except (IOError, RuntimeError):
            pass

    def test_open_missing_file(self):
        filename = "___missing_file___.___"
        image_format = jpeg2kimage.Jpeg2KImage()
        try:
            _image = image_format.read(filename)
            self.fail()
        except IOError:
            pass


class TestJpeg2KImage_PIL(TestJpeg2KImage):
    """Test the class format using a specific decoder"""

    def setUp(self):
        if not isPilUsable():
            self.skipTest("PIL is not available or has no support for JPEG2000")

    @classmethod
    def setUpClass(cls):
        # Remove other decoders
        cls.old = jpeg2kimage.glymur
        jpeg2kimage.glymur = None

    @classmethod
    def tearDownClass(cls):
        # Remove other decoders
        jpeg2kimage.glymur = cls.old
        cls.old = None


class TestJpeg2KImage_glymur(TestJpeg2KImage):
    """Test the class format using a specific decoder"""

    def setUp(self):
        if not isGlymurUsable():
            self.skipTest("glymur is not available or too old")

    @classmethod
    def setUpClass(cls):
        # Remove other decoders
        cls.old = jpeg2kimage.PIL
        jpeg2kimage.PIL = None

    @classmethod
    def tearDownClass(cls):
        # Remove other decoders
        jpeg2kimage.PIL = cls.old
        cls.old = None


class TestJpeg2KImage_fabio(TestJpeg2KImage):
    """Test the format inside the fabio framework"""

    def loadImage(self, filename):
        """Use the fabio API instead of using the image format"""
        image = fabio.open(filename)
        return image


def suite():
    loader = unittest.defaultTestLoader.loadTestsFromTestCase
    testsuite = unittest.TestSuite()
    testsuite.addTest(loader(TestJpeg2KImage))
    testsuite.addTest(loader(TestJpeg2KImage_PIL))
    testsuite.addTest(loader(TestJpeg2KImage_glymur))
    testsuite.addTest(loader(TestJpeg2KImage_fabio))
    return testsuite


if __name__ == "__main__":
    runner = unittest.TextTestRunner()
    runner.run(suite())



# coding: utf-8
#
#    Project: X-ray image reader
#             https://github.com/silx-kit/fabio
#
#
#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
#
#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#  .
#  The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#  .
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#  THE SOFTWARE.

"""
Authors: Gael Goret, Jerome Kieffer, ESRF, France

Emails: gael.goret@esrf.fr, jerome.kieffer@esrf.fr
        Brian Richard Pauw <brian@stack.nl>

Binary files images are simple none-compressed 2D images only defined by their :
data-type, dimensions, byte order and offset

This simple library has been made for manipulating exotic/unknown files format.
"""

__authors__ = ["Gaël Goret", "Jérôme Kieffer", "Brian Pauw"]
__contact__ = "gael.goret@esrf.fr"
__license__ = "MIT"
__copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
__date__ = "13/03/2026"

import io
from .fabioimage import FabioImage
import numpy
import logging

logger = logging.getLogger(__name__)


class BinaryImage(FabioImage):
    """
    This simple library has been made for manipulating exotic/unknown files format.

    Binary files images are simple none-compressed 2D images only defined by their:
        data-type, dimensions, byte order and offset

    if offset is set to a negative value, the image is read using the last data but n
    data in the file, skipping any header.
    """

    DESCRIPTION = "Binary format (none-compressed 2D images)"

    DEFAULT_EXTENSIONS = ["bin"]

    def __init__(self, *args, **kwargs):
        FabioImage.__init__(self, *args, **kwargs)

    @staticmethod
    def swap_needed(endian):
        """
        Decide if we need to byteswap
        """
        if (endian == "<" and numpy.little_endian) or (
            endian == ">" and not numpy.little_endian
        ):
            return False
        if (endian == ">" and numpy.little_endian) or (
            endian == "<" and not numpy.little_endian
        ):
            return True

    def read(self, fname, dim1, dim2, offset=0, bytecode="int32", endian="<"):
        """
        Read a binary image

        :param str fname: file name
        :param int dim1: image dimensions (Fast index)
        :param int dim2: image dimensions (Slow index)
        :param int offset: starting position of the data-block. If negative, starts at the end.
        :param bytecode: can be "int8","int16","int32","int64","uint8","uint16","uint32","uint64","float32","float64",...
        :param endian:  among little or big endian ("<" or ">")

        """
        assert endian in ("<", ">", "=")
        bytecode = numpy.dtype(bytecode)
        if not bytecode.str.startswith(endian):
            bytecode = numpy.dtype(endian + bytecode.str[1:])
        self.filename = fname
        self._shape = dim2, dim1
        self._bytecode = bytecode
        with open(self.filename, "rb") as f:
            dims = [dim2, dim1]
            bpp = numpy.dtype(bytecode).itemsize
            size = dims[0] * dims[1] * bpp

            if offset >= 0:
                f.seek(offset)
            else:
                try:
                    f.seek(-size + offset + 1, 2)  # seek from EOF backwards
                except IOError:
                    logger.warning(
                        "expected datablock too large, please check bytecode settings: {}".format(
                            bytecode
                        )
                    )
                except Exception:
                    logger.debug("Backtrace", exc_info=True)
                    logger.error("Uncommon error encountered when reading file")
            rawData = f.read(size)
        data = numpy.frombuffer(rawData, bytecode).copy().reshape(tuple(dims))
        self.data = data
        self._shape = None
        return self

    def estimate_offset_value(self, fname, dim1, dim2, bytecode="int32"):
        "Estimates the size of a file"
        with open(fname, "rb") as f:
            bpp = len(numpy.array(0, bytecode).tobytes())
            size = dim1 * dim2 * bpp
            totsize = len(f.read())
        logger.info("total size (bytes): %s", totsize)
        logger.info("expected data size given parameters (bytes): %s", size)
        logger.info("estimation of the offset value (bytes): %s", totsize - size)

    def write(self, fname):
        with self._open(fname, mode="wb") as outfile:
            if isinstance(outfile, io.BufferedWriter):
                self.data.tofile(outfile)
            else:
                outfile.write(self.data.tobytes())


binaryimage = BinaryImage



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Project: Fable Input Output
#             https://github.com/silx-kit/fabio
#
#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
#
#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#

"""
Unit tests for the d*TREK format.
"""

import unittest
import os
import logging
import numpy
import shutil
from ..utilstest import UtilsTest
import fabio
from fabio.dtrekimage import DtrekImage
from fabio.edfimage import EdfImage
from fabio.utils import testutils

logger = logging.getLogger(__name__)

# statistics come from fit2d I think
# filename dim1 dim2 min max mean stddev
TESTIMAGES = [
    (
        "mb_LP_1_001.img",
        (3072, 3072),
        0.0000,
        65535.0,
        120.33,
        147.38,
        {"BEAM_CENTER_Y": "157.500000"},
    ),
    (
        "mb_LP_1_001.img.gz",
        (3072, 3072),
        0.0000,
        65535.0,
        120.33,
        147.38,
        {"BEAM_CENTER_Y": "157.500000"},
    ),
    (
        "mb_LP_1_001.img.bz2",
        (3072, 3072),
        0.0000,
        65535.0,
        120.33,
        147.38,
        {"BEAM_CENTER_Y": "157.500000"},
    ),
    (
        "HSA_1_5mg_C1_0004.img",
        (385, 775),
        -2,
        2127,
        69.25,
        59.52,
        {"WAVELENGTH": "1.0 1.541870"},
    ),
]


class TestMatch(unittest.TestCase):
    """
    check the ??fit2d?? conversion to edf gives same numbers
    """

    def setUp(self):
        """Download images"""
        self.fn_adsc = UtilsTest.getimage("mb_LP_1_001.img.bz2")[:-4]
        self.fn_edf = UtilsTest.getimage("mb_LP_1_001.edf.bz2")[:-4]

    def testsame(self):
        """test ADSC image match to EDF"""
        im1 = EdfImage()
        im1.read(self.fn_edf)
        im2 = DtrekImage()
        im2.read(self.fn_adsc)
        diff = im1.data.astype("float32") - im2.data.astype("float32")
        logger.debug(
            "type: %s %s shape %s %s "
            % (im1.data.dtype, im2.data.dtype, im1.data.shape, im2.data.shape)
        )
        logger.debug(
            "im1 min %s %s max %s %s "
            % (im1.data.min(), im2.data.min(), im1.data.max(), im2.data.max())
        )
        logger.debug(
            "delta min %s max %s mean %s" % (diff.min(), diff.max(), diff.mean())
        )
        self.assertEqual(abs(diff).max(), 0.0, "asdc data == edf data")


class TestDtrekImplementation(testutils.ParametricTestCase):
    @classmethod
    def setUpClass(cls):
        cls.tmp_directory = os.path.join(UtilsTest.tempdir, cls.__name__)
        os.makedirs(cls.tmp_directory)

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.tmp_directory)

    def setUp(self):
        unittest.TestCase.setUp(self)

    def test_write_and_read_cube(self):
        input_type = numpy.uint16
        data = numpy.arange(5 * 10 * 2).reshape(5, 10, 2)
        data = data.astype(input_type)
        obj = DtrekImage(data=data)
        filename = os.path.join(self.tmp_directory, "cube.img")
        obj.save(filename)
        self.assertEqual(obj.data.dtype.type, input_type)
        obj2 = fabio.open(filename)
        self.assertEqual(obj2.data.dtype.type, input_type)
        self.assertEqual(obj.shape, obj2.shape)
        numpy.testing.assert_array_almost_equal(obj.data, obj2.data)

    def test_write_and_read_empty(self):
        obj = DtrekImage(data=None)
        filename = os.path.join(self.tmp_directory, "cube.img")
        obj.save(filename)
        obj2 = fabio.open(filename)
        self.assertEqual(obj2.data, None)
        self.assertEqual(obj.data, obj2.data)

    def test_write_and_read(self):
        configs = [
            (numpy.uint16, "little_endian", None),
            (numpy.uint16, "big_endian", None),
            (numpy.uint32, "little_endian", None),
            (numpy.uint32, "big_endian", None),
            (numpy.int32, "little_endian", None),
            (numpy.int32, "big_endian", None),
            (numpy.float32, "little_endian", None),
            (numpy.float32, "big_endian", None),
            # Data have to be converted before storage
            (numpy.uint64, "little_endian", numpy.uint32),
            (numpy.int64, "little_endian", numpy.int32),
            (numpy.float16, "little_endian", numpy.float32),
        ]
        for config in configs:
            with self.subTest(config=config):
                input_type, byte_order, output_type = config
                if output_type is None:
                    output_type = input_type

                header = {}
                header["BYTE_ORDER"] = byte_order
                data = numpy.arange(5 * 10).reshape(5, 10)
                data = data.astype(input_type)
                obj = DtrekImage(data=data, header=header)
                filename = os.path.join(
                    self.tmp_directory, "saved_%s.img" % hash(config)
                )
                obj.save(filename)
                self.assertEqual(obj.data.dtype.type, input_type)
                obj2 = fabio.open(filename)
                self.assertEqual(obj2.data.dtype.type, output_type)
                self.assertEqual(obj.shape, obj2.shape)
                if input_type == output_type:
                    numpy.testing.assert_array_almost_equal(obj.data, obj2.data)


class TestRealSamples(testutils.ParametricTestCase):
    """
    Test real samples stored in our archive.
    """

    @classmethod
    def setUpClass(cls):
        """Prefetch images"""
        download = []
        for datainfo in TESTIMAGES:
            name = datainfo[0]
            if name.endswith(".bz2"):
                download.append(name)
            elif name.endswith(".gz"):
                download.append(name[:-3] + ".bz2")
            else:
                download.append(name + ".bz2")
        download = list(set(download))
        for name in download:
            os.path.dirname(UtilsTest.getimage(name))
        cls.im_dir = UtilsTest.resources.data_home

    def test_read(self):
        """check we can read flat ADSC images"""
        for datainfo in TESTIMAGES:
            with self.subTest(datainfo=datainfo):
                name, shape, mini, maxi, mean, stddev, keys = datainfo
                obj = DtrekImage()
                obj.read(os.path.join(self.im_dir, name))
                self.assertAlmostEqual(mini, obj.getmin(), 2, "getmin")
                self.assertAlmostEqual(maxi, obj.getmax(), 2, "getmax")
                got_mean = obj.getmean()
                self.assertAlmostEqual(
                    mean, got_mean, 2, "getmean exp %s != got %s" % (mean, got_mean)
                )
                self.assertAlmostEqual(stddev, obj.getstddev(), 2, "getstddev")
                for key, value in keys.items():
                    self.assertIn(key, obj.header)
                    self.assertEqual(value, obj.header[key])
                self.assertEqual(shape, obj.shape)


def suite():
    loadTests = unittest.defaultTestLoader.loadTestsFromTestCase
    testsuite = unittest.TestSuite()
    testsuite.addTest(loadTests(TestMatch))
    testsuite.addTest(loadTests(TestRealSamples))
    testsuite.addTest(loadTests(TestDtrekImplementation))
    return testsuite


if __name__ == "__main__":
    runner = unittest.TextTestRunner()
    runner.run(suite())



#!/usr/bin/env python

"""
Test IO size limits
"""

import os
import unittest
import logging
import numpy
import tempfile
import shutil
import fabio.edfimage
import fabio.tifimage

logger = logging.getLogger(__name__)


class _CommonIOLimitTest(unittest.TestCase):
    IMAGETYPE = None
    HEADERSIZE = 0

    @classmethod
    def setUpClass(cls):
        cls.test_dir = tempfile.mkdtemp()
        cls.test_filename = os.path.join(cls.test_dir, "data")

        # A single IO operation is limited to `nbytes_single_io` bytes
        nbytes_single_io = 2**31
        cls.data_dtype = numpy.uint32
        itemsize = 4

        # Make sure we will need more than one IO operation
        n = int((nbytes_single_io / itemsize) ** 0.5) + 1
        cls.data_shape = n, n
        cls.data_size = n * n
        cls.data_size_bytes = n * n * itemsize
        cls.save_data()

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.test_dir)

    @classmethod
    def generate_data(cls):
        return (
            numpy.arange(cls.data_size).reshape(cls.data_shape).astype(cls.data_dtype)
        )

    @classmethod
    def save_data(cls):
        if cls.IMAGETYPE is None:
            raise unittest.SkipTest("no image type specified")
        if logger.getEffectiveLevel() > logging.DEBUG:
            raise unittest.SkipTest("test requires debug level logging")
        cls.IMAGETYPE(data=cls.generate_data()).write(cls.test_filename)

    @classmethod
    def filesize(cls):
        return os.stat(cls.test_filename).st_size

    def test_filesize(self):
        expected = self.data_size_bytes + self.HEADERSIZE
        self.assertEqual(self.filesize(), expected)

    def test_io_roundtrip(self):
        image = self.IMAGETYPE()
        data = image.read(self.test_filename).data
        self.assertEqual(self.data_shape, data.shape)
        self.assertEqual(self.data_dtype, data.dtype)
        self.assertEqual(data[0, 0], 0)
        self.assertEqual(data[-1, -1], self.data_size - 1)


class TestEdf(_CommonIOLimitTest):
    IMAGETYPE = fabio.edfimage.edfimage
    HEADERSIZE = 512


class TestTiff(_CommonIOLimitTest):
    IMAGETYPE = fabio.tifimage.tifimage
    HEADERSIZE = 196



def suite():
    loadTests = unittest.defaultTestLoader.loadTestsFromTestCase
    testsuite = unittest.TestSuite()
    testsuite.addTest(loadTests(TestEdf))
    testsuite.addTest(loadTests(TestTiff))
    return testsuite


if __name__ == "__main__":
    runner = unittest.TextTestRunner()
    runner.run(suite())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Project: Fast Azimuthal integration
#             https://github.com/silx-kit/pyFAI
#
#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
#
#    Authors: Gael Goret <gael.goret@esrf.fr>
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions.
#  .
#  The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#  .
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
Portable diffraction images viewer/converter
* Written in Python, it combines the functionalities of the I/O library fabIO
  with a user-friendly Qt GUI.
* Image converter is also a light viewer based on the visualization tool
  provided by the module matplotlib.
"""

# ----------------------------------------------------------------------
# Imports (QtPy, FabIO, NumPy, Matplotlib, …)
# ----------------------------------------------------------------------
import numpy
from qtpy import QtWidgets as qt
from qtpy.QtWidgets import (
    QMessageBox,
    QLabel,
    QDialog,
    QDialogButtonBox,
    QGroupBox,
)

numpy.seterr(divide="ignore")
output_format = [
    "*.bin",
    "*.cbf",
    "*.edf",
    "*.h5",
    "*.img",
    "*.mar2300",
    "*.mar3450",
    "*.marccd",
    "*.tiff",
    "*.sfrm",
]


# ──────────────────────────────────────────────────────────────────────
#   Dialog windows – now fully English
# ──────────────────────────────────────────────────────────────────────
class CounterFormatOptionDialog(QDialog):
    """Dialog that lets the user set the file‑counter format used when saving
    a series of files."""

    def __init__(self, counter_format, parent=None):
        super().__init__(parent)
        self.setWindowTitle("Counter format")
        self.resize(350, 100)
        # ----- layout ----------------------------------------------------
        button_box = QDialogButtonBox(self)
        button_box.setGeometry(qt.QRect(0, 60, 341, 32))
        button_box.setOrientation(qt.Qt.Horizontal)
        button_box.setStandardButtons(QDialogButtonBox.Cancel | QDialogButtonBox.Ok)
        label = QLabel("File counter format:", self)
        label.setGeometry(qt.QRect(38, 23, 181, 16))
        self.lineEdit = qt.QLineEdit(self)
        self.lineEdit.setGeometry(qt.QRect(175, 18, 113, 25))
        self.lineEdit.setText(counter_format)
        # ----- signals ---------------------------------------------------
        button_box.accepted.connect(self.accept)
        button_box.rejected.connect(self.reject)

    def exec_(self):
        """Return the new format string if the user clicks *Ok*,
        otherwise return the original value."""
        if super().exec_() == QDialog.Accepted:
            new_text = str(self.lineEdit.text()).strip()
            if new_text:
                return new_text
            QMessageBox.warning(
                self,
                "Warning",
                "The counter format cannot be empty – the previous value will be kept.",
            )
        return self.lineEdit.text()


class DownSamplingDialog(QDialog):
    """Dialog that gathers the parameters required for down‑sampling
    (summation) of a stack of images."""

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowTitle("Down‑sampling")
        self.resize(410, 250)
        # ----- layout ----------------------------------------------------
        button_box = QDialogButtonBox(self)
        button_box.setGeometry(qt.QRect(45, 200, 341, 32))
        button_box.setOrientation(qt.Qt.Horizontal)
        button_box.setStandardButtons(QDialogButtonBox.Cancel | QDialogButtonBox.Ok)
        # Number of files to sum
        label_n = QLabel("Number of files to sum:", self)
        label_n.setGeometry(qt.QRect(38, 63, 181, 16))
        self.lineEdit_n = qt.QLineEdit(self)
        self.lineEdit_n.setGeometry(qt.QRect(220, 58, 113, 25))
        # Starting phi angle
        label_phi0 = QLabel("Starting φ angle (degrees):", self)
        label_phi0.setGeometry(qt.QRect(90, 100, 191, 20))
        self.lineEdit_phi0 = qt.QLineEdit(self)
        self.lineEdit_phi0.setGeometry(qt.QRect(300, 95, 113, 25))
        # Phi step
        label_step = QLabel("φ step (degrees):", self)
        label_step.setGeometry(qt.QRect(151, 133, 101, 16))
        self.lineEdit_step = qt.QLineEdit(self)
        self.lineEdit_step.setGeometry(qt.QRect(260, 130, 113, 25))
        # ----- signals ---------------------------------------------------
        button_box.accepted.connect(self.accept)
        button_box.rejected.connect(self.reject)

    def exec_(self):
        """Return a tuple *(thick, start_angle, step_angle)* if the user
        confirms, otherwise *(None, None, None)*."""
        if super().exec_() == QDialog.Accepted:
            if (
                self.lineEdit_n.text()
                and self.lineEdit_phi0.text()
                and self.lineEdit_step.text()
            ):
                try:
                    thick = int(self.lineEdit_n.text())
                    start_angle = float(self.lineEdit_phi0.text())
                    step_angle = float(self.lineEdit_step.text())
                except ValueError:
                    QMessageBox.warning(
                        self,
                        "Invalid input",
                        "Please enter numeric values for all fields.",
                    )
                    return None, None, None
                return thick, start_angle, step_angle
            QMessageBox.warning(
                self,
                "Missing data",
                "All fields are mandatory – please fill them before confirming.",
            )
        return None, None, None


class BinDialog(QDialog):
    """Dialog that asks the user for the parameters needed to read a raw
    binary data block (dimensions, offset, byte‑code, endian)."""

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowTitle("Binary data block opening")
        self.resize(410, 270)
        # ----- internal state --------------------------------------------
        self.dim1 = None
        self.dim2 = None
        self.offset = None
        self.bytecode = None
        self.endian = None
        # ----- layout ----------------------------------------------------
        button_box = QDialogButtonBox(self)
        button_box.setGeometry(qt.QRect(50, 230, 341, 32))
        button_box.setOrientation(qt.Qt.Horizontal)
        button_box.setStandardButtons(QDialogButtonBox.Cancel | QDialogButtonBox.Ok)
        group_box = QGroupBox("Binary data block required information:", self)
        group_box.setGeometry(qt.QRect(10, 10, 370, 191))
        # Dimension 1
        label_dim1 = QLabel("Dimension 1:", group_box)
        label_dim1.setGeometry(qt.QRect(67, 48, 91, 16))
        self.lineEdit_dim1 = qt.QLineEdit(group_box)
        self.lineEdit_dim1.setGeometry(qt.QRect(185, 40, 91, 25))
        # Dimension 2
        label_dim2 = QLabel("Dimension 2:", group_box)
        label_dim2.setGeometry(qt.QRect(66, 76, 91, 16))
        self.lineEdit_dim2 = qt.QLineEdit(group_box)
        self.lineEdit_dim2.setGeometry(qt.QRect(185, 70, 91, 25))
        # Offset
        label_offset = QLabel("Offset:", group_box)
        label_offset.setGeometry(qt.QRect(105, 106, 51, 16))
        self.lineEdit_offset = qt.QLineEdit(group_box)
        self.lineEdit_offset.setGeometry(qt.QRect(184, 100, 91, 25))
        self.lineEdit_offset.setText("0")
        # Byte code
        label_bc = QLabel("Byte code:", group_box)
        label_bc.setGeometry(qt.QRect(70, 130, 91, 16))
        self.comboBox_bc = qt.QComboBox(group_box)
        self.comboBox_bc.setGeometry(qt.QRect(173, 123, 91, 25))
        for bc in [
            "int8",
            "int16",
            "int32",
            "int64",
            "uint8",
            "uint16",
            "uint32",
            "uint64",
            "float32",
            "float64",
        ]:
            self.comboBox_bc.addItem(bc)
        self.comboBox_bc.setCurrentIndex(2)  # default to int32
        # Endian
        label_endian = QLabel("Endian:", group_box)
        label_endian.setGeometry(qt.QRect(98, 170, 61, 16))
        self.comboBox_endian = qt.QComboBox(group_box)
        self.comboBox_endian.setGeometry(qt.QRect(182, 166, 91, 25))
        self.comboBox_endian.addItem("Little‑endian")
        self.comboBox_endian.addItem("Big‑endian")
        # ----- signals ---------------------------------------------------
        button_box.accepted.connect(self._accept)
        button_box.rejected.connect(self.reject)

    # ------------------------------------------------------------------
    def _accept(self):
        """Validate the entries and store them before closing the dialog."""
        if (
            self.lineEdit_dim1.text()
            and self.lineEdit_dim2.text()
            and self.lineEdit_offset.text()
        ):
            try:
                self.dim1 = int(self.lineEdit_dim1.text())
                self.dim2 = int(self.lineEdit_dim2.text())
                self.offset = int(self.lineEdit_offset.text())
            except ValueError:
                QMessageBox.warning(
                    self,
                    "Invalid input",
                    "Dimensions and offset must be integer numbers.",
                )
                return
        else:
            QMessageBox.warning(
                self,
                "Missing data",
                "All fields are mandatory – please fill them before confirming.",
            )
            return
        self.bytecode = str(self.comboBox_bc.currentText())
        # QtPy returns the displayed text, which we map back to the original
        # “Short” / “Long” terminology used later in the code.
        self.endian = (
            "Short" if self.comboBox_endian.currentText() == "Little‑endian" else "Long"
        )
        self.accept()

    # ------------------------------------------------------------------
    def exec_(self):
        """Return the 5‑tuple *(dim1, dim2, offset, bytecode, endian)*
        if the user clicks **Ok**, otherwise a tuple of ``None``."""
        if super().exec_() == QDialog.Accepted:
            return (
                self.dim1,
                self.dim2,
                self.offset,
                self.bytecode,
                self.endian,
            )
        return None, None, None, None, None



# coding: utf-8
#
#    Project: X-ray image reader
#             https://github.com/silx-kit/fabio
#
#
#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
#
#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#  .
#  The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#  .
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#  THE SOFTWARE.

"""
Test module for each codecs supported by FabIO
"""

__authors__ = ["Jérôme Kieffer"]
__contact__ = "jerome.kieffer@esrf.eu"
__license__ = "MIT+"
__copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
__date__ = "15/06/2026"


import unittest


def suite():
    from . import test_edfimage
    from . import test_edfimage_expg
    from . import test_cbfimage
    from . import test_dtrekimage
    from . import test_fit2dmaskimage
    from . import test_fit2dspreadsheetimage
    from . import test_geimage
    from . import test_mar345image
    from . import test_brukerimage
    from . import test_bruker100image
    from . import test_mccdimage
    from . import test_oxdimage
    from . import test_kcdimage
    from . import test_tifimage
    from . import test_xsdimage
    from . import test_raxisimage
    from . import test_pnmimage
    from . import test_numpyimage
    from . import test_pilatusimage
    from . import test_eigerimage
    from . import test_lambdaimage
    from . import test_hdf5image
    from . import test_fit2dimage
    from . import test_speimage
    from . import test_jpegimage
    from . import test_jpeg2kimage
    from . import test_mpaimage
    from . import test_dm3image
    from . import test_mrcimage
    from . import test_pixiimage
    from . import test_esperantoimage
    from . import test_limaimage
    from . import test_hipicimage
    from . import test_binaryimage
    from . import test_xcaliburimage

    testSuite = unittest.TestSuite()
    testSuite.addTest(test_edfimage.suite())
    testSuite.addTest(test_edfimage_expg.suite())
    testSuite.addTest(test_cbfimage.suite())
    testSuite.addTest(test_dtrekimage.suite())
    testSuite.addTest(test_fit2dmaskimage.suite())
    testSuite.addTest(test_fit2dspreadsheetimage.suite())
    testSuite.addTest(test_geimage.suite())
    testSuite.addTest(test_mar345image.suite())
    testSuite.addTest(test_brukerimage.suite())
    testSuite.addTest(test_bruker100image.suite())
    testSuite.addTest(test_mccdimage.suite())
    testSuite.addTest(test_oxdimage.suite())
    testSuite.addTest(test_kcdimage.suite())
    testSuite.addTest(test_tifimage.suite())
    testSuite.addTest(test_xsdimage.suite())
    testSuite.addTest(test_raxisimage.suite())
    testSuite.addTest(test_pnmimage.suite())
    testSuite.addTest(test_numpyimage.suite())
    testSuite.addTest(test_pilatusimage.suite())
    testSuite.addTest(test_eigerimage.suite())
    testSuite.addTest(test_hdf5image.suite())
    testSuite.addTest(test_fit2dimage.suite())
    testSuite.addTest(test_speimage.suite())
    testSuite.addTest(test_jpegimage.suite())
    testSuite.addTest(test_jpeg2kimage.suite())
    testSuite.addTest(test_mpaimage.suite())
    testSuite.addTest(test_dm3image.suite())
    testSuite.addTest(test_mrcimage.suite())
    testSuite.addTest(test_pixiimage.suite())
    testSuite.addTest(test_esperantoimage.suite())
    testSuite.addTest(test_limaimage.suite())
    testSuite.addTest(test_hipicimage.suite())
    testSuite.addTest(test_binaryimage.suite())
    testSuite.addTest(test_xcaliburimage.suite())
    testSuite.addTest(test_lambdaimage.suite())
    return testSuite


def run_tests():
    """Run test complete test_suite"""
    runner = unittest.TextTestRunner()
    if not runner.run(suite()).wasSuccessful():
        print("Test suite failed")
        return 1
    else:
        print("Test suite succeeded")
        return 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Project: Fable Input Output
#             https://github.com/silx-kit/fabio
#
#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
#
#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
#
#  Permission is hereby granted, free of charge, to any person
#  obtaining a copy of this software and associated documentation files
#  (the "Software"), to deal in the Software without restriction,
#  including without limitation the rights to use, copy, modify, merge,
#  publish, distribute, sublicense, and/or sell copies of the Software,
#  and to permit persons to whom the Software is furnished to do so,
#  subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be
#  included in all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
#  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
#  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
#  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
#  OTHER DEALINGS IN THE SOFTWARE.

__authors__ = ["Jérôme Kieffer"]
__contact__ = "Jerome.Kieffer@esrf.fr"
__license__ = "MIT"
__copyright__ = "2020 ESRF"
__date__ = "27/10/2025"

import unittest
import numpy
import logging
from ..sparseimage import densify, cython_densify
from ..ext.dense import distribution_uniform_mtc, distribution_normal_mtc

logger = logging.getLogger(__name__)


class TestDensification(unittest.TestCase):
    def test_rng_uniform(self):
        shape = (100, 100)
        U = distribution_uniform_mtc(shape).ravel()
        d, p = numpy.histogram(U, 100)
        eps = 1e-3
        self.assertGreater(p[0], 0)
        self.assertLess(p[0], eps)
        self.assertGreater(p[-1], 1 - eps)
        self.assertLess(p[-1], 1)
        self.assertGreater(d.min(), 50)
        self.assertLess(d.max(), 150)

    def test_rng_normal(self):
        shape = (100, 100)
        mu = 5.5
        sigma = 1.5
        one = numpy.ones(shape)
        N = distribution_normal_mtc(one * mu, one * sigma)
        self.assertAlmostEqual(N.mean(), mu, 1)
        self.assertAlmostEqual(N.std(), sigma, 1)

    def test_cython(self):
        seed = 0
        shape = 256, 256
        nframes = 8
        vsize = 181  # This is cheated to avoid interpolation issues with rounding 128*sqrt(2)
        y, x = numpy.ogrid[
            -shape[0] // 2 : -shape[0] // 2 + shape[0],
            -shape[1] // 2 : -shape[1] // 2 + shape[1],
        ]
        # To make this test "robust", those two radial position arrays needs to be in float64 ... in production float32 is more common
        r2d = numpy.sqrt(x * x + y * y).astype(numpy.float64)
        radius = numpy.linspace(0, r2d.max(), vsize).astype(numpy.float64)
        npeak = numpy.random.randint(90, 110, size=nframes)
        scale = numpy.random.randint(90, 110, size=nframes)
        osc = numpy.random.randint(40, 100, size=nframes)
        indptr = numpy.zeros(nframes + 1, dtype=int)
        indptr[1:] = numpy.cumsum(npeak)
        index = numpy.random.randint(0, numpy.prod(shape), size=npeak.sum()).astype(
            numpy.uint32
        )
        intensity = numpy.random.randint(0, 10, size=npeak.sum()).astype(numpy.uint16)
        frames = numpy.empty((nframes, *shape), dtype=numpy.uint16)
        background = numpy.empty((nframes, vsize), dtype=numpy.float32)
        noise = numpy.empty((nframes, vsize), dtype=numpy.float32)
        python = []
        cython = []
        for i, f in enumerate(frames):
            background[i] = numpy.arcsinh(scale[i] * numpy.sinc(radius / osc[i]) ** 2)
            noise[i] = abs(numpy.diff(background[i], prepend=background[i, 0]))
            f[...] = numpy.arcsinh(scale[i] * numpy.sinc(r2d / osc[i]) ** 2).round()
            f.ravel()[index[indptr[i] : indptr[i + 1]]] = intensity[
                indptr[i] : indptr[i + 1]
            ]
            python = densify(
                r2d,
                radius,
                index[indptr[i] : indptr[i + 1]],
                intensity[indptr[i] : indptr[i + 1]],
                0,
                background[i],
            )
            cython = cython_densify.densify(
                r2d,
                radius,
                index[indptr[i] : indptr[i + 1]],
                intensity[indptr[i] : indptr[i + 1]],
                0,
                intensity.dtype,
                background[i],
            )
            if not numpy.all(python == cython):
                print(python)
                print(cython)
            self.assertTrue(numpy.all(python == cython), "python == cython #" + str(i))
            # Rounding errors:
            delta = python.astype(int) - f
            self.assertLessEqual(
                abs(delta).max(), 1, "Maximum difference is 1 due to rounding errors"
            )
            bad = numpy.where(delta)
            print("#####", i)
            self.assertLess(
                len(bad[0]),
                numpy.prod(shape) / 500,
                "python differs from reference on less then 0.2% of the pixel #"
                + str(i),
            )

            # Now consider the noise ...
            python = densify(
                r2d,
                radius,
                index[indptr[i] : indptr[i + 1]],
                intensity[indptr[i] : indptr[i + 1]],
                0,
                background[i],
                noise[i],
                seed=seed,
            )
            cython = cython_densify.densify(
                r2d,
                radius,
                index[indptr[i] : indptr[i + 1]],
                intensity[indptr[i] : indptr[i + 1]],
                0,
                intensity.dtype,
                background[i],
                noise[i],
                seed=seed,
            )
            self.assertTrue(
                abs(python.astype(int) - cython).max() <= 2 * max(1, noise[i].max()),
                "python is close to cython #" + str(i),
            )


def suite():
    loadTests = unittest.defaultTestLoader.loadTestsFromTestCase
    testsuite = unittest.TestSuite()
    testsuite.addTest(loadTests(TestDensification))
    return testsuite


if __name__ == "__main__":
    runner = unittest.TextTestRunner()
    runner.run(suite())
